#!/usr/bin/python
#
#  setup.py - [ install script for mini-dinstall ]
#  Copyright (c) 2007 Christoph Goehre <christoph.goehre@gmx.de>
#
#  This file is part of the mini-dinstall package.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.
#

from distutils.core import setup

setup(name = 'mini-dinstall',
      author = 'Colin Walters',
      author_email = 'walters@debian.org',
      maintainer = 'Christoph Goehre',
      maintainer_email = 'christoph.goehre@gmx.de',
      scripts = [ 'mini-dinstall' ],
      packages = [ 'minidinstall' ],
      url = 'http://alioth.debian.org/projects/mini-dinstall/',
)
